@@ -13,12 +13,15 @@ from datahub.exceptions import DatahubException
 from datahub.models import BlobRecord
 from datahub.models import CursorType
 from datahub import DataHub
+import fastredis
 import traceback
+import datetime
 import urllib3
 import showlog
+import json
 import envx
 import sys
-import datetime
+
 # datahub安装时使用pydatahub
 
 """
@@ -552,3 +555,78 @@ def topic_list(
     return basic.topic_list(
         project_name=project_name
     )
+
+
+def lazy_save(
+        data: dict,
+        save_project_name: str,
+        save_topic_name: str,
+        save_env_file_name_datahub: str,
+        save_env_file_name_redis: str,
+        save_redis_db: int,
+        project_auto_create: bool = True,
+        topic_auto_create: bool = True
+):
+    """
+    此模块提供了存储json数据到DataHub的功能
+    如果存储错误，将暂存在redis中，由错误处理进程继续处理
+
+    :param data: 需要存储的数据，dict类型，单条存储
+    :param save_project_name: 存储的目标project
+    :param save_topic_name: 存储的目标topic
+    :param save_env_file_name_datahub: datahub环境文件
+    :param save_env_file_name_redis: redis环境文件
+    :param save_redis_db: redis缓存数据库
+    :param project_auto_create: 自动创建project
+    :param topic_auto_create: 自动创建topic
+    """
+    save_res = {
+        "code": 0,
+        "msg": "ok",
+        "success": True
+    }
+    try:
+        while True:
+            showlog.info('正在保存数据到DataHub...')
+            add_response = blob_records_add(
+                project_name=save_project_name,
+                topic_name=save_topic_name,
+                blob_data=json.dumps(data),
+                env_file_name=save_env_file_name_datahub,
+                project_auto_create=project_auto_create,
+                topic_auto_create=topic_auto_create
+            )  # 以blob方式存储，更加灵活
+            add_response_code = add_response.get('code')
+            if add_response_code == 0:
+                showlog.info('保存成功')
+                return save_res
+            elif add_response_code == 404:
+                message = add_response.get('message')
+                save_res['code'] = add_response_code
+                save_res['msg'] = message
+                save_res['success'] = False
+                return save_res
+            else:
+                fastredis.list_add_r(
+                    key=save_topic_name,
+                    value=json.dumps(data),
+                    env_file_name=save_env_file_name_redis,
+                    db=save_redis_db
+                )
+                return save_res
+    except:
+        try:
+            # 出错的转存到redis
+            fastredis.list_add_r(
+                key=save_topic_name,
+                value=json.dumps(data),
+                env_file_name=save_env_file_name_redis,
+                db=save_redis_db
+            )
+        except:
+            save_res = {
+                "code": 1,
+                "msg": "failed",
+                "success": False
+            }
+    return save_res
